function handles = getLabelBrush(img, fig, ax, labels, select, handles)


[rows cols dim] = size(img);

if(nargin ==3 || isempty(labels))
    labels = zeros(rows, cols);
    
else
    si = size(img);
    sl = size(labels);
    assert(all(sl(1:2)==si(1:2)),...
         'labels and image must be the same size');
end

if(nargin < 5 || nargin >= 5 && strcmp(select,'select') )
    select = 1; %true
end

radius = 6;

if(select == 1) 

    set(fig, 'WindowButtonDownFcn', {@brush_gui_onmousedown});
    set(fig, 'WindowButtonUpFcn', {@brush_gui_onmouseup});
    set(fig, 'WindowButtonMotionFcn', {@brush_gui_onmousemove});
else

    
    set(fig, 'WindowButtonDownFcn', {@erase_gui_onmousedown});
    set(fig, 'WindowButtonUpFcn', {@erase_gui_onmouseup});
    set(fig, 'WindowButtonMotionFcn', {@erase_gui_onmousemove});
end


    
cp = [];
mouse_down = 0;
handles.labels = labels;

numargs = nargin;

value = 1;

  function brush_gui_onmousedown(hObject, event) %#ok<INUSD>
    % Fired when the user presses the mouse button.
        %make sure no interruption while backing up
        
        set(hObject,'Interruptible','off');
        handles = guidata(hObject);

        handles = backup_state(hObject, handles);
        guidata(hObject, handles);
        
        
         
        cp = get(ax,'CurrentPoint');
       
        x = cp(1,1);
        y = cp(1,2);
        
         click = get(fig,'SelectionType');
         if(strcmp(click, 'normal')) %left click,'normal'
            value = 1;
         else
            value = -1; %right click
         end
     
        brush_gui_select(x,y,hObject, handles)
       set(hObject,'Interruptible','on');
    end

    function brush_gui_onmouseup(hObject,event) %#ok<INUSD>
    % Fired when the user releases the mouse button.
        %set(hObject,'Interruptible','off');
        handles = guidata(hObject);
        cp = [];  % no currently selected point
        
              
    end

    function brush_gui_onmousemove(hObject, event)
    % Fired when the user moves the mouse.
        if isempty(cp)  % no point selected (not in drag mode)
            return;
        end
        handles = guidata(hObject); 
       
        cp = get(ax,'CurrentPoint');
        x = cp(1,1);
        y = cp(1,2);
     
        [rows cols dim] = size(handles.img);
        
       if( ~isempty(x) && (x<0 || y < 0 || y> rows || x > cols) )
        disp('click outside the img, label not counted');
        
        return;
       end
       
       %set(hObject,'Interruptible','off');
        handles = backup_state(hObject, handles);
        guidata(hObject, handles);
        %set(hObject,'Interruptible','on');
        
       brush_gui_select(x,y,hObject, handles)
       
       
    end

    function  brush_gui_select(x,y,hObject, handles)
    % Update current selection.
        if(~isempty(x))
            handles.labels(round(y),round(x)) = value;
            guidata(hObject,handles);
        end
        if(value == 1)
            type = 'r.';
        else
            type = 'y.';
        end
        
        %{
           foreground = line(-1, -1, ...
            'Parent', ax, ...
            'Color', 'r', ...
            'Marker', '.',...
            'MarkerSize', 20,...
            'LineWidth', 2,...
            'LineStyle','none');
            background = line(-1, -1, ...
            'Parent', ax, ...
            'Color', 'y', ...
            'Marker', '.',...
            'MarkerSize', 20,...
            'LineWidth', 2,...
            'LineStyle','none');
%}
            [r c] = find(handles.labels==1);
            %set(foreground, 'XData', c, 'YData', r);
           % set(foreground, 'Visible', 'on');
     
            
            
            [r c] = find(handles.labels==-1);
            
            %set(background, 'XData', c, 'YData', r);
            %set(background, 'Visible', 'on');
            
               
            %clear foreground;
            %clear background;
            
            hold on;
            hp = plot(x,y,type,'MarkerSize',22);
            set(hp, 'Interruptible','off');
        
    end

    function erase_gui_onmousedown(hObject, event) %#ok<INUSD>
    % Fired when the user presses the mouse button.
        %make sure no interruption while backing up
        set(hObject,'Interruptible','off');
        handles = guidata(hObject);

        handles = backup_state(hObject, handles);
        
        
        
        cp = get(ax,'CurrentPoint');
       
        x = cp(1,1);
        y = cp(1,2);
        
        value = 0;
        
        [xv yv] = circle([x,y], radius,50,'c.');
        [y x] = find(handles.labels==1 | handles.labels==-1);
        in = inpolygon(x, y, xv, yv);
        handles.labels( y(in), x(in) ) = value;
        
        %imshow(handles.img);
        guidata(hObject, handles);
        %brush_gui_select([],[],hObject, handles)
        mouse_down = 1;
        set(hObject,'Interruptible','on');
        
    end

    function erase_gui_onmouseup(hObject,event) %#ok<INUSD>
    % Fired when the user releases the mouse button.
        handles = guidata(hObject);
        mouse_down = 0;
       
        display_img(handles);

    end

    function erase_gui_onmousemove(hObject, event)
        handles = guidata(hObject);
        
     
        cp = get(ax,'CurrentPoint');
        
        x = cp(1,1);
        y = cp(1,2);
        
        

        if(isfield(handles,'selection'))
            try
                set(handles.selection,'Visible', 'off');
                handles = rmfield(handles,'selection');
            catch
                %do nothing
            end
        end
        
        handles.selection = line(0, 0, ...
        'Parent', ax, ...
        'Color', 'g', ...
        'Marker', 'o',...
        'MarkerSize', radius*2+2,...
        'LineWidth', 2);
        
    
        
        set(handles.selection,'XData',x,'YData',y,'Visible', 'on');
        set(handles.selection, 'HandleVisibility','on');
        
        if(mouse_down == 1)
            [xv yv] = circle([x,y], radius,50,'c.');
            [y x] = find(handles.labels==1 | handles.labels==-1);
            in = inpolygon(x, y, xv, yv);
            handles.labels( y(in), x(in) ) = value;
            
        end
        
        guidata(hObject, handles);
        
    end
end


