%Script gives a sample usage of the random walker function for image
%segmentation
%
%
%10/31/05 - Leo Grady

clear
close all

%Read image
%img=im2double(imread('axial_CT_slice.bmp'));
img = im2double(imread('test.jpg'));
%img = imresize(img, .5);
[X Y]=size(img);

%Set two seeds
%s1x=130; s1y=150; %Note that seed location is not central to object
%s2x=200; s2y=200;

figure;
imshow(img, 'border','tight');
disp('click for label 1');
[s1x s1y] = ginput; %foreground
disp('click for label 2');
[s2x s2y] = ginput; %background
disp('click for label 3');
[s3x s3y] = ginput; %other area

s1x =round(s1x);
s1y =round(s1y);
s2x = round(s2x);
s2y = round(s2y);
s3x = round(s3x);
s3y = round(s3y);

index1 = sub2ind([X Y],s1y,s1x)';
index2 = sub2ind([X Y],s2y,s2x)';
index3 = sub2ind([X Y],s3y,s3x)';
labels = [repmat(1,size(index1)) ... 
          repmat(2,size(index2)) ...
          repmat(3,size(index3)) ];


%Apply the random walker algorithms
%[mask,probabilities] = random_walker(img,[sub2ind([X Y],s1y,s1x),...
%    sub2ind([X Y],s2y,s2x)],[1,2]);

[mask,probabilities] = random_walker(img,[index1 index2 index3], labels);

%Display results
figure
imagesc(img);
colormap('gray')
axis equal
axis tight
axis off
hold on
plot(s1x,s1y,'g.','MarkerSize',24)
plot(s2x,s2y,'b.','MarkerSize',24)
plot(s3x,s3y,'r.', 'MarkerSize',24)
title('Image with foreground (green) and background (blue) seeds')

figure
imagesc(mask)
colormap('gray')
axis equal
axis tight
axis off
hold on
plot(s1x,s1y,'g.','MarkerSize',24)
plot(s2x,s2y,'b.','MarkerSize',24)
plot(s3x,s3y,'r.', 'MarkerSize',24)
title('Output mask');

figure
[imgMasks,segOutline,imgMarkup]=segoutput(img,mask);
imagesc(imgMarkup);
colormap('gray')
axis equal
axis tight
axis off
hold on
plot(s1x,s1y,'g.','MarkerSize',24)
plot(s2x,s2y,'b.','MarkerSize',24)
plot(s3x,s3y,'r.', 'MarkerSize',24)
title('Outlined mask')

figure
imagesc(probabilities(:,:,1))
colormap('gray')
axis equal
axis tight
axis off
hold on
plot(s1x,s1y,'g.','MarkerSize',24)
plot(s2x,s2y,'b.','MarkerSize',24)
plot(s3x,s3y,'r.', 'MarkerSize',24)
title(strcat('Probability at each pixel that a random walker released ', ...
    'from that pixel reaches the foreground seed'));